<?php 

namespace PrestaShop\Module\LCVPrestaConnector\Controller;

use PrestaShop\Module\LCVPrestaConnector\Forms\GrilleType;
use PrestaShop\Module\LCVPrestaConnector\Forms\InformationType;
use Symfony\Component\HttpFoundation\Request;
use PrestaShop\Module\LCVPrestaConnector\SyncConfiguration;
use PrestaShop\Module\LCVPrestaConnector\Syncer;

class ConfigurationAdminController extends AdminController
{
    /**
     * Indique si la resynchronisation est en cours
     */
    private ?bool $isResyncing = null;

    /**
     * Obtient la description de la barre d'outils de l'interface d'administration
     */
    private function getAdminLayoutToolBar() : array 
    {
        $toolbar = [
                'bind' => [
                    'href' => $this->generateUrl($this->module->name . '_configuration') . '&reset-binding=1',
                    'class' => 'btn-outline-secondary',
                    'desc' => 'Déconnecter',
                    'icon' => 'cancel',                    
                ],
                'resync' => [
                    'href' => '#',
                    'class' => 'btn-outline-primary',
                    'desc' => 'Resynchroniser',
                    'icon' => 'refresh',                    
                    'js' => $this->module->name . '_resyncAll();',
                ],
            ];
                
        if ($this->isResyncing === null)
        {
            // Extraction de la position de synchronisation
            $positions = $this->module->getCfg()->get(SyncConfiguration::CFG_SYNC_POSITIONS);
            $this->isResyncing = empty($positions);
        }

        if ($this->isResyncing)
        {
            $toolbar['resync']['desc'] = 'Resynchro. demandée !';
            $toolbar['resync']['class'] = 'btn-outline-secondary disabled';
        }

        return $toolbar;
    }

    /**
     * Retourne les variables passées en paramètre enrichies des variables par défaut
     */
    private function addDefaultVars(array $vars) : array
    {
        if (!isset($vars['layoutHeaderToolbarBtn']))
            $vars['layoutHeaderToolbarBtn'] = $this->getAdminLayoutToolBar();

        if (!isset($vars['backoffice']))
            $vars['backoffice'] = $this->module->getBridge()->getId();
        
        return $vars;
    }


    public function grilleDeTaille(Request $request){
     
        $form = $this->createForm(GrilleType::class);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            //$data = $form->getData();
        }
    
        return $form;

    }

    public function informationForm(Request $request){

        $form = $this->createForm(InformationType::class);

        $form->handleRequest($request);

        if($form->isSubmitted() && $form->isValid()){
            //on recupere la valeur du notre switch apres validation du formulaire 
            $mode = $form->get("cfg_mode")->getData();
            //on enregistre directement dans la bdd si mode est vrai alors la valeur et à 0 sinon elle est à 1 
            $this->module->getCfg()->set(SyncConfiguration::CFG_WORKMODE,  $mode ? 0 : 1 ); 

            $this->addFlash('success', 'Configuration sauvegardée !');
        }

        return $form;

    }


    /**
     * Vérifie que le backoffice est bien jumelé ou renvoi null
     */
    public function checkLogin() : \Symfony\Component\HttpFoundation\Response|null
    {
        $loginCfg = $this->module->getCfg()->get(SyncConfiguration::CFG_LOGIN);

        if (empty($loginCfg) || (isset($loginCfg["error"]) && $loginCfg["error"]))
        {
            $specifics = $this->module->getBridge()->getSpecifics();            
            
            return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/login.html.twig', [
                'MODULE_NAME' => $this->module->name,
                'BACKOFFICE_NAME' => $this->module->getBridge()->getId(),
                'ADDRESS_LABEL' => $specifics["ADDRESS_LABEL"] ?? "",
                'PREVIOUS_ADDRESS' => $specifics["PREVIOUS_ADDRESS"] ?? "",
                "INITIAL_ERROR" => (isset($loginCfg["error"]) && $loginCfg["error"]) ? $loginCfg["error"] : "Il semble que le module n'est pas encore associé avec votre backoffice.",
            ]);            
        }

        return null;
    }

    public function informationAction(Request $request){

        // On vérifie s'il faut remonter les hooks
        if (isset($_REQUEST["remount_hook"]))
			$this->module->registerHooks();

        // On vérifie s'il faut réinitialisé le jumelage
        if (isset($_REQUEST["reset-binding"]) && $_REQUEST["reset-binding"])
        {
            // Oui, on réinitialise le jumelage en supprimant la configuration de login
            $cfgLogin = null;
            $this->module->getCfg()->set(SyncConfiguration::CFG_LOGIN, $cfgLogin);
        }

        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;

        // On vérifie s'il faut programmer une resynchronisation totale
        if (isset($_REQUEST["reset-resync"]) && $_REQUEST["reset-resync"])
        {
            // On instancie un syncer
            $syncer = new Syncer($this->module);

            // Demande de resynchronisation totale pour la prochaine fois
            $syncer->bridge->resetSyncPosition();
        }

        $form = $this->informationForm($request);
        
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/information.html.twig', $this->addDefaultVars([
            'form' => $form->createView()
        ]));
    }

    public function optionAction(){
        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;
  
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/option.html.twig', $this->addDefaultVars([
            ]));
    }

    public function commandeAction(){
        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;
  
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/commande.html.twig', [
            'layoutHeaderToolbarBtn' => $this->getAdminLayoutToolBar(), 
        ]);
    }


    public function catalogueAction(Request $request){
        // Vérifions que le jumelage est déjà fait !
        $bindPage = $this->checkLogin(); if ($bindPage) return $bindPage;

        $catalogOption = $this->module->getCfg()->get(SyncConfiguration::CFG_DATA_SYNC);
    
    
  
        return $this->render('@Modules/' . $this->module->name . '/views/templates/admin/catalogue.html.twig', $this->addDefaultVars([
            'optionsDesc'=> $this->module->getCfg()->describeSyncConfiguration(),
            'optionsValues'=> $catalogOption
        ]));
    }


   

}